@@ -26,7 +26,7 @@ class Link(ABC):
         return super().__new__(GenericLink)
         #raise LinkTypeUnknown()
 
-    def __init__(self, name, link, ethtool):
+    def __init__(self, name, link, ethtool, vrrp):
         self.cap_create = True
         self.cap_ethtool = False
         self.settings = {
@@ -34,6 +34,7 @@ class Link(ABC):
         }
         self.settings.update(link)
         self.ethtool = None
+        self.vrrp = vrrp
         self.attr_map = {
             'kind': ['IFLA_LINKINFO', 'IFLA_INFO_KIND'],
         }
@@ -42,8 +43,15 @@ class Link(ABC):
         self.idx = None
 
         if 'address' in self.settings:
+            self.settings['address'] = self.settings['address'].lower()
             self.idx = next(iter(ipr.link_lookup(
                 address=self.settings['address'])), None)
+        if 'permaddr' in self.settings:
+            self.settings['permaddr'] = self.settings['permaddr'].lower()
+            self.idx = ipr.get_iface_by_permaddr(self.settings['permaddr'])
+        if 'businfo' in self.settings:
+            self.settings['businfo'] = self.settings['businfo'].lower()
+            self.idx = ipr.get_iface_by_businfo(self.settings['businfo'])
 
     def _drill_attr(self, data, keys):
         key = keys[0]
@@ -58,8 +66,11 @@ class Link(ABC):
             return None
 
     def get_if_attr(self, key):
-        if key == "state":
-            return self.iface['state']
+        if key in ["state", "permaddr", "businfo"]:
+            if key in self.iface:
+                return self.iface[key]
+            else:
+                return None
 
         if key in self.attr_map:
             return self._drill_attr(self.iface, self.attr_map[key])
@@ -161,6 +172,15 @@ class Link(ABC):
             except Exception as err:
                 logger.warning('failed write `{}`: {}'.format(fn, err.args[1]))
 
+    def has_vrrp(self):
+        return not self.vrrp is None
+
+    def match_vrrp_select(self, vrrp_type, vrrp_name):
+        return self.has_vrrp() and (self.vrrp['type'] == vrrp_type) and (self.vrrp['name'] == vrrp_name)
+
+    def match_vrrp_state(self, vrrp_type, vrrp_name, vrrp_state):
+        return self.match_vrrp_select(vrrp_type, vrrp_name) and (vrrp_state in self.vrrp['states'])
+
     def apply(self, do_apply):
         excpts = ExceptionCollector()
         osettings = copy.deepcopy(self.settings)
@@ -171,11 +191,18 @@ class Link(ABC):
                 self.settings[attr] = next(iter(ipr.link_lookup(
                     ifname=self.settings[attr])), self.settings[attr])
 
-        self.idx = next(iter(ipr.link_lookup(
-            ifname=self.settings['ifname'])), None)
+        if self.idx is None:
+            self.idx = next(iter(ipr.link_lookup(
+                ifname=self.settings['ifname'])), None)
 
         if self.idx is not None:
             self.iface = next(iter(ipr.get_links(self.idx)), None)
+            permaddr = ipr.get_permaddr(self.iface.get_attr('IFLA_IFNAME'))
+            if not permaddr is None:
+                self.iface['permaddr'] = permaddr
+            businfo = ipr.get_businfo(self.iface.get_attr('IFLA_IFNAME'))
+            if not businfo is None:
+                self.iface['businfo'] = businfo
 
             # check for ifname collisions
             idx = next(iter(ipr.link_lookup(
@@ -360,5 +387,5 @@ class Link(ABC):
 
 
 class GenericLink(Link):
-    def __init__(self, name, link, ethtool):
-        super().__init__(name, link, ethtool)
+    def __init__(self, name, link, ethtool, vrrp):
+        super().__init__(name, link, ethtool, vrrp)
